@@ -82,14 +82,14 @@ typedef struct GC_ms_entry * (*GC_mark_proc)(GC_word * /* addr */,
 
 /* Object descriptors on mark stack or in objects.  Low order two       */
 /* bits are tags distinguishing among the following 4 possibilities     */
-/* for the high order 30 bits.                                          */
+/* for the rest (high order) bits.                                      */
 #define GC_DS_TAG_BITS 2
 #define GC_DS_TAGS   ((1 << GC_DS_TAG_BITS) - 1)
 #define GC_DS_LENGTH 0  /* The entire word is a length in bytes that    */
                         /* must be a multiple of 4.                     */
-#define GC_DS_BITMAP 1  /* 30 (62) bits are a bitmap describing pointer */
-                        /* fields.  The msb is 1 if the first word      */
-                        /* is a pointer.                                */
+#define GC_DS_BITMAP 1  /* The high order bits are describing pointer   */
+                        /* fields.  The most significant bit is set if  */
+                        /* the first word is a pointer.                 */
                         /* (This unconventional ordering sometimes      */
                         /* makes the marker slightly faster.)           */
                         /* Zeroes indicate definite nonpointers.  Ones  */
@@ -101,7 +101,7 @@ typedef struct GC_ms_entry * (*GC_mark_proc)(GC_word * /* addr */,
                         /* PROC(descr).  ENV(descr) is passed as the    */
                         /* last argument.                               */
 #define GC_MAKE_PROC(proc_index, env) \
-            (((((env) << GC_LOG_MAX_MARK_PROCS) \
+            ((((((GC_word)(env)) << GC_LOG_MAX_MARK_PROCS) \
                | (proc_index)) << GC_DS_TAG_BITS) | GC_DS_PROC)
 #define GC_DS_PER_OBJECT 3  /* The real descriptor is at the            */
                         /* byte displacement from the beginning of the  */
@@ -245,7 +245,7 @@ GC_API GC_ATTR_MALLOC GC_ATTR_ALLOC_SIZE(1) void * GC_CALL
 #endif /* !GC_DEBUG */
 
 /* Similar to GC_size but returns object kind.  Size is returned too    */
-/* if psize is not NULL.                                                */
+/* if psize is not NULL.  The object pointer should not be NULL.        */
 GC_API int GC_CALL GC_get_kind_and_size(const void *, size_t * /* psize */)
                                                         GC_ATTR_NONNULL(1);
 
